
/*
 * uPD7810 assembly parser
 *
 * EXx ... BLOCK and CMC opcodes had inconsistencies in docco. Beware.
 * CMC non-existent ?
 *
 * Expect to see 28 shift/reduce conflicts from yacc,
 * caused by rule r containing A.
 * Some conflicts were dodged by inlining parts
 * of rX rules (see MOV). Yucko, not nice, but works.
 */

word		: expr {
				$$ = $1;
				if (pass && (unsigned) $$ >= 65536)
					if (((unsigned) $$ >> 16) != 0xFFFF) /* accept ~0xNNNN */
						yyerror("word out of range");
			}
			;

byte		: expr {
				$$ = $1;
				if (pass && (unsigned) $$ >= 256)
					if (((unsigned) $$ >> 8) != 0xFFFFFF) /* accept ~0xNN */
						yyerror("byte out of range");
			}
			;

bit			: expr {
				$$ = $1;
				if (pass && (unsigned) $$ >= 8)
					yyerror("bit number out of range");
			}
			;

addr		: '[' word ']' { $$ = $2; }
immw		: word
immb		: byte
wa			: byte
			;

bcdehl		: B		{ $$ = 0x02; }
			| C		{ $$ = 0x03; }
			| D		{ $$ = 0x04; }
			| E		{ $$ = 0x05; }
			| H		{ $$ = 0x06; }
			| L		{ $$ = 0x07; }
			;

r			: V		{ $$ = 0x00; }
			| A		{ $$ = 0x01; }
			| bcdehl
			;

r1			: EAH	{ $$ = 0x00; }
			| EAL	{ $$ = 0x01; }
			;

r2			: A		{ $$ = 0x01; }
			| B		{ $$ = 0x02; }
			| C		{ $$ = 0x03; }
			;

sr			: PA	{ $$ = 0x00; }
			| PB	{ $$ = 0x01; }
			| PC	{ $$ = 0x02; }
			| PD	{ $$ = 0x03; }
			| PF	{ $$ = 0x05; }
			| MKH	{ $$ = 0x06; }
			| MKL	{ $$ = 0x07; }
			| ANM	{ $$ = 0x08; }
			| SMH	{ $$ = 0x09; }
			| SML	{ $$ = 0x0A; }
			| EOM	{ $$ = 0x0B; }
			| ETMM	{ $$ = 0x0C; }
			| TMM	{ $$ = 0x0D; }
			| MM	{ $$ = 0x10; }
			| MCC	{ $$ = 0x11; }
			| MA	{ $$ = 0x12; }
			| MB	{ $$ = 0x13; }
			| MC	{ $$ = 0x14; }
			| MF	{ $$ = 0x17; }
			| TXB	{ $$ = 0x18; }
			| TM0	{ $$ = 0x1A; }
			| TM1	{ $$ = 0x1B; }
			| ZCM	{ $$ = 0x28; }
			;

sr1			: PA	{ $$ = 0x00; }
			| PB	{ $$ = 0x01; }
			| PC	{ $$ = 0x02; }
			| PD	{ $$ = 0x03; }
			| PF	{ $$ = 0x05; }
			| MKH	{ $$ = 0x06; }
			| MKL	{ $$ = 0x07; }
			| ANM	{ $$ = 0x08; }
			| SMH	{ $$ = 0x09; }
			| EOM	{ $$ = 0x0B; }
			| TMM	{ $$ = 0x0D; }
			| RXB	{ $$ = 0x19; }
			| CR0	{ $$ = 0x20; }
			| CR1	{ $$ = 0x21; }
			| CR2	{ $$ = 0x22; }
			| CR3	{ $$ = 0x23; }
			;

sr2			: PA	{ $$ = 0x00; }
			| PB	{ $$ = 0x01; }
			| PC	{ $$ = 0x02; }
			| PD	{ $$ = 0x03; }
			| PF	{ $$ = 0x05; }
			| MKH	{ $$ = 0x06; }
			| MKL	{ $$ = 0x07; }
			| ANM	{ $$ = 0x08; }
			| SMH	{ $$ = 0x09; }
			| EOM	{ $$ = 0x0B; }
			| TMM	{ $$ = 0x0D; }
			;

sr3			: ETM0	{ $$ = 0x00; }
			| ETM1	{ $$ = 0x01; }
			;

sr4			: ECNT	{ $$ = 0x00; }
			| ECPT	{ $$ = 0x01; }
			;

rp			: SP	{ $$ = 0x00; }
			| BC	{ $$ = 0x01; }
			| DE	{ $$ = 0x02; }
			| HL	{ $$ = 0x03; }
			;

rp1			: VA	{ $$ = 0x00; }
			| BC	{ $$ = 0x01; }
			| DE	{ $$ = 0x02; }
			| HL	{ $$ = 0x03; }
			| EA	{ $$ = 0x04; }
			;

rp2			: SP	{ $$ = 0x00; }
			| BC	{ $$ = 0x01; }
			| DE	{ $$ = 0x02; }
			| HL	{ $$ = 0x03; }
			| EA	{ $$ = 0x04; }
			;

rp3			: BC	{ $$ = 0x01; }
			| DE	{ $$ = 0x02; }
			| HL	{ $$ = 0x03; }
			;

rpa			: '[' BC ']'		{ $$ = 0x01; }
			| '[' DE ']'		{ $$ = 0x02; }
			| '[' HL ']'		{ $$ = 0x03; }
			| '[' DE ']' '+'	{ $$ = 0x04; }
			| '[' HL ']' '+'	{ $$ = 0x05; }
			| '[' DE ']' '-'	{ $$ = 0x06; }
			| '[' HL ']' '-'	{ $$ = 0x07; }
			;

rpa1		: '[' BC ']'		{ $$ = 0x01; }
			| '[' DE ']'		{ $$ = 0x02; }
			| '[' HL ']'		{ $$ = 0x03; }
			;

rpa2		: '[' BC ']'			{ $$ = 0x01; }
			| '[' DE ']'			{ $$ = 0x02; }
			| '[' HL ']'			{ $$ = 0x03; }
			| '[' DE ']' '+'		{ $$ = 0x04; }
			| '[' HL ']' '+'		{ $$ = 0x05; }
			| '[' DE ']' '-'		{ $$ = 0x06; }
			| '[' HL ']' '-'		{ $$ = 0x07; }
			| '[' DE '+' byte ']'	{ $$ = 0x0B | ($4 << 8) | 0x10000; }
			| '[' HL '+' A ']'		{ $$ = 0x0C; }
			| '[' HL '+' B ']'		{ $$ = 0x0D; }
			| '[' HL '+' EA ']'		{ $$ = 0x0E; }
			| '[' HL '+' byte ']'	{ $$ = 0x0F | ($4 << 8) | 0x10000; }
			;

rpa3		: '[' DE ']'			{ $$ = 0x02; }
			| '[' HL ']'			{ $$ = 0x03; }
			| '[' DE ']' '+' '+'	{ $$ = 0x04; }
			| '[' HL ']' '+' '+'	{ $$ = 0x05; }
			| '[' DE '+' byte ']'	{ $$ = 0x0B | ($4 << 8) | 0x10000; }
			| '[' HL '+' A ']'		{ $$ = 0x0C; }
			| '[' HL '+' B ']'		{ $$ = 0x0D; }
			| '[' HL '+' EA ']'		{ $$ = 0x0E; }
			| '[' HL '+' byte ']'	{ $$ = 0x0F | ($4 << 8) | 0x10000; }
			;

f			: CY		{ $$ = 0x02; }
			| HC		{ $$ = 0x03; }
			| Z			{ $$ = 0x04; }
			;

irf			: INTFNMI	{ $$ = 0x00; }
			| INTFT0	{ $$ = 0x01; }
			| INTFT1	{ $$ = 0x02; }
			| INTF1		{ $$ = 0x03; }
			| INTF2		{ $$ = 0x04; }
			| INTFE0	{ $$ = 0x05; }
			| INTFE1	{ $$ = 0x06; }
			| INTFEIN	{ $$ = 0x07; }
			| INTFAD	{ $$ = 0x08; }
			| INTFSR	{ $$ = 0x09; }
			| INTFST	{ $$ = 0x0A; }
			| ER		{ $$ = 0x0B; }
			| OV		{ $$ = 0x0C; }
			| AN4		{ $$ = 0x10; }
			| AN5		{ $$ = 0x11; }
			| AN6		{ $$ = 0x12; }
			| AN7		{ $$ = 0x13; }
			| SB		{ $$ = 0x14; }
			;

instruction	: MOV r1 ',' A     { op1(b00011000 | $2); }
			| MOV bcdehl ',' A { op1(b00011000 | $2); }
			| MOV A ',' r1     { op1(b00001000 | $4); }
			| MOV A ',' bcdehl { op1(b00001000 | $4); }

			| MOV sr ',' A   { op2(b01001101, b11000000 | $2); }
			| MOV A ',' sr1  { op2(b01001100, b11000000 | $4); }

			| MOV V      ',' addr { op4(b01110000, b01101000 | 0, $4, $4 >> 8); }
			| MOV A      ',' addr { op4(b01110000, b01101000 | 1, $4, $4 >> 8); }
			| MOV bcdehl ',' addr { op4(b01110000, b01101000 | $2, $4, $4 >> 8); }
			| MOV addr ',' V      { op4(b01110000, b01111000 | 0, $2, $2 >> 8); }
			| MOV addr ',' A      { op4(b01110000, b01111000 | 1, $2, $2 >> 8); }
			| MOV addr ',' bcdehl { op4(b01110000, b01111000 | $4, $2, $2 >> 8); }

			| MVI r ',' immb   { op2(b01101000 | $2, $4); }
			| MVI sr2 ',' immb { op3(b01100100, (($2 & 8) << 4) | ($2 & 7), $4); }

			| MVIW wa ',' immb   { op3(b01110001, $2, $4); }
			| MVIX rpa1 ',' immb { op2(b01001000 | $2, $4); }

			| STAW wa { op2(b01100011, $2); }
			| LDAW wa { op2(b00000001, $2); }

			| STAX rpa2  {
				if ($2 & 0x10000)
					op2(b00111000 | (($2 & 8) << 4) | ($2 & 7), $2 >> 8);
				else
					op1(b00111000 | (($2 & 8) << 4) | ($2 & 7));
			}
			| LDAX rpa2  {
				if ($2 & 0x10000)
					op2(b00101000 | (($2 & 8) << 4) | ($2 & 7), $2 >> 8);
				else
					op1(b00101000 | (($2 & 8) << 4) | ($2 & 7));
			}
			| STEAX rpa3 {
				if ($2 & 0x10000)
					op3(b01001000, b10010000 | $2, $2 >> 8);
				else
					op2(b01001000, b10010000 | $2);
			}
			| LDEAX rpa3 {
				if ($2 & 0x10000)
					op3(b01001000, b10000000 | $2, $2 >> 8);
				else
					op2(b01001000, b10000000 | $2);
			}

			| EXX   { op1(b00010001); }
			| EXA   { op1(b00010000); }
			| EXH   { op1(b01010000); }
			| BLOCK { op1(b00110001); }

			| DMOV rp3 ',' EA { op1(b10110100 | $2); }
			| DMOV EA ',' rp3 { op1(b10100100 | $4); }

			| DMOV sr3 ',' EA { op2(b01001000, b11010010 | $2); }
			| DMOV EA ',' sr4 { op2(b01001000, b11000000 | $4); }

			| SSPD addr { op4(b01110000, b00001110, $2, $2 >> 8); }
			| SBCD addr { op4(b01110000, b00011110, $2, $2 >> 8); }
			| SDED addr { op4(b01110000, b00101110, $2, $2 >> 8); }
			| SHLD addr { op4(b01110000, b00111110, $2, $2 >> 8); }

			| LSPD addr { op4(b01110000, b00001111, $2, $2 >> 8); }
			| LBCD addr { op4(b01110000, b00011111, $2, $2 >> 8); }
			| LDED addr { op4(b01110000, b00101111, $2, $2 >> 8); }
			| LHLD addr { op4(b01110000, b00111111, $2, $2 >> 8); }

			| PUSH rp1 { op1(b10110000 | $2); }
			| POP  rp1 { op1(b10100000 | $2); }

			| LXI rp2 ',' immw { op3(b00000100 | ($2 << 4), $4, $4 >> 8); }

			| TABLE  { op2(b01001000, b10101000); }

			| ADD A ',' r   { op2(b01100000, b11000000 | $4); }
			| ADD r ',' A   { op2(b01100000, b01000000 | $2); }
			| ADC A ',' r   { op2(b01100000, b11010000 | $4); }
			| ADC r ',' A   { op2(b01100000, b01010000 | $2); }
			| ADDNC A ',' r { op2(b01100000, b10100000 | $4); canskip(); }
			| ADDNC r ',' A { op2(b01100000, b00100000 | $2); canskip(); }
			| SUB A ',' r   { op2(b01100000, b11100000 | $4); }
			| SUB r ',' A   { op2(b01100000, b01100000 | $2); }
			| SBB A ',' r   { op2(b01100000, b11110000 | $4); }
			| SBB r ',' A   { op2(b01100000, b01110000 | $2); }
			| SUBNB A ',' r { op2(b01100000, b10110000 | $4); canskip(); }
			| SUBNB r ',' A { op2(b01100000, b00110000 | $2); canskip(); }
			| ANA A ',' r   { op2(b01100000, b10001000 | $4); }
			| ANA r ',' A   { op2(b01100000, b00001000 | $2); }
			| ORA A ',' r   { op2(b01100000, b10011000 | $4); }
			| ORA r ',' A   { op2(b01100000, b00011000 | $2); }
			| XRA A ',' r   { op2(b01100000, b10010000 | $4); }
			| XRA r ',' A   { op2(b01100000, b00010000 | $2); }
			| GTA A ',' r   { op2(b01100000, b10101000 | $4); canskip(); }
			| GTA r ',' A   { op2(b01100000, b00101000 | $2); canskip(); }
			| LTA A ',' r   { op2(b01100000, b10111000 | $4); canskip(); }
			| LTA r ',' A   { op2(b01100000, b00111000 | $2); canskip(); }
			| NEA A ',' r   { op2(b01100000, b11101000 | $4); canskip(); }
			| NEA r ',' A   { op2(b01100000, b01101000 | $2); canskip(); }
			| EQA A ',' r   { op2(b01100000, b11111000 | $4); canskip(); }
			| EQA r ',' A   { op2(b01100000, b01111000 | $2); canskip(); }
			| ONA A ',' r   { op2(b01100000, b11001000 | $4); canskip(); }
			| OFFA A ',' r  { op2(b01100000, b11011000 | $4); canskip(); }

			| ADDX   rpa { op2(b01110000, b11000000 | $2); }
			| ADCX   rpa { op2(b01110000, b11010000 | $2); }
			| ADDNCX rpa { op2(b01110000, b10100000 | $2); canskip(); }
			| SUBX   rpa { op2(b01110000, b11100000 | $2); }
			| SBBX   rpa { op2(b01110000, b11110000 | $2); }
			| SUBNBX rpa { op2(b01110000, b10110000 | $2); canskip(); }
			| ANAX   rpa { op2(b01110000, b10001000 | $2); }
			| ORAX   rpa { op2(b01110000, b10011000 | $2); }
			| XRAX   rpa { op2(b01110000, b10010000 | $2); }
			| GTAX   rpa { op2(b01110000, b10101000 | $2); canskip(); }
			| LTAX   rpa { op2(b01110000, b10111000 | $2); canskip(); }
			| NEAX   rpa { op2(b01110000, b11101000 | $2); canskip(); }
			| EQAX   rpa { op2(b01110000, b11111000 | $2); canskip(); }
			| ONAX   rpa { op2(b01110000, b11001000 | $2); canskip(); }
			| OFFAX  rpa { op2(b01110000, b11011000 | $2); canskip(); }

			| ADI   A ',' immb   { op2(b01000110, $4); }
			| ADI   r ',' immb   { op3(b01110100, b01000000 | $2, $4); }
			| ADI   sr2 ',' immb { op3(b01100100, b01000000 | (($2 & 8) << 4) | ($2 & 7), $4); }
			| ACI   A ',' immb   { op2(b01010110, $4); }
			| ACI   r ',' immb   { op3(b01110100, b01010000 | $2, $4); }
			| ACI   sr2 ',' immb { op3(b01100100, b01010000 | (($2 & 8) << 4) | ($2 & 7), $4); }
			| ADINC A ',' immb   { op2(b00100110, $4); canskip(); }
			| ADINC r ',' immb   { op3(b01110100, b00100000 | $2, $4); canskip(); }
			| ADINC sr2 ',' immb { op3(b01100100, b00100000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }
			| SUI   A ',' immb   { op2(b01100110, $4); }
			| SUI   r ',' immb   { op3(b01110100, b01100000 | $2, $4); }
			| SUI   sr2 ',' immb { op3(b01100100, b01100000 | (($2 & 8) << 4) | ($2 & 7), $4); }
			| SBI   A ',' immb   { op2(b01110110, $4); }
			| SBI   r ',' immb   { op3(b01110100, b01110000 | $2, $4); }
			| SBI   sr2 ',' immb { op3(b01100100, b01110000 | (($2 & 8) << 4) | ($2 & 7), $4); }
			| SUINB A ',' immb   { op2(b00110110, $4); canskip(); }
			| SUINB r ',' immb   { op3(b01110100, b00110000 | $2, $4); canskip(); }
			| SUINB sr2 ',' immb { op3(b01100100, b00110000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }
			| ANI   A ',' immb   { op2(b00000111, $4); }
			| ANI   r ',' immb   { op3(b01110100, b00001000 | $2, $4); }
			| ANI   sr2 ',' immb { op3(b01100100, b00001000 | (($2 & 8) << 4) | ($2 & 7), $4); }
			| ORI   A ',' immb   { op2(b00010111, $4); }
			| ORI   r ',' immb   { op3(b01110100, b00011000 | $2, $4); }
			| ORI   sr2 ',' immb { op3(b01100100, b00011000 | (($2 & 8) << 4) | ($2 & 7), $4); }
			| XRI   A ',' immb   { op2(b00010110, $4); }
			| XRI   r ',' immb   { op3(b01110100, b00010000 | $2, $4); }
			| XRI   sr2 ',' immb { op3(b01100100, b00010000 | (($2 & 8) << 4) | ($2 & 7), $4); }
			| GTI   A ',' immb   { op2(b00100111, $4); canskip(); }
			| GTI   r ',' immb   { op3(b01110100, b00101000 | $2, $4); canskip(); }
			| GTI   sr2 ',' immb { op3(b01100100, b00101000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }
			| LTI   A ',' immb   { op2(b00110111, $4); canskip(); }
			| LTI   r ',' immb   { op3(b01110100, b00111000 | $2, $4); canskip(); }
			| LTI   sr2 ',' immb { op3(b01100100, b00111000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }
			| NEI   A ',' immb   { op2(b01100111, $4); canskip(); }
			| NEI   r ',' immb   { op3(b01110100, b01101000 | $2, $4); canskip(); }
			| NEI   sr2 ',' immb { op3(b01100100, b01101000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }
			| EQI   A ',' immb   { op2(b01110111, $4); canskip(); }
			| EQI   r ',' immb   { op3(b01110100, b01111000 | $2, $4); canskip(); }
			| EQI   sr2 ',' immb { op3(b01100100, b01111000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }
			| ONI   A ',' immb   { op2(b01000111, $4); canskip(); }
			| ONI   r ',' immb   { op3(b01110100, b01001000 | $2, $4); canskip(); }
			| ONI   sr2 ',' immb { op3(b01100100, b01001000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }
			| OFFI  A ',' immb   { op2(b01010111, $4); canskip(); }
			| OFFI  r ',' immb   { op3(b01110100, b01011000 | $2, $4); canskip(); }
			| OFFI  sr2 ',' immb { op3(b01100100, b01011000 | (($2 & 8) << 4) | ($2 & 7), $4); canskip(); }

			| ADDW   wa { op3(b01110100, b11000000, $2); }
			| ADCW   wa { op3(b01110100, b11010000, $2); }
			| ADDNCW wa { op3(b01110100, b10100000, $2); canskip(); }
			| SUBW   wa { op3(b01110100, b11100000, $2); }
			| SBBW   wa { op3(b01110100, b11110000, $2); }
			| SUBNBW wa { op3(b01110100, b10110000, $2); canskip(); }
			| ANAW   wa { op3(b01110100, b10001000, $2); }
			| ORAW   wa { op3(b01110100, b10011000, $2); }
			| XRAW   wa { op3(b01110100, b10010000, $2); }
			| GTAW   wa { op3(b01110100, b10101000, $2); canskip(); }
			| LTAW   wa { op3(b01110100, b10111000, $2); canskip(); }
			| NEAW   wa { op3(b01110100, b11101000, $2); canskip(); }
			| EQAW   wa { op3(b01110100, b11111000, $2); canskip(); }
			| ONAW   wa { op3(b01110100, b11001000, $2); canskip(); }
			| OFFAW  wa { op3(b01110100, b11011000, $2); canskip(); }

			| ANIW   wa ',' immb { op3(b00000101, $2, $4); }
			| ORIW   wa ',' immb { op3(b00010101, $2, $4); }
			| GTIW   wa ',' immb { op3(b00100101, $2, $4); canskip(); }
			| LTIW   wa ',' immb { op3(b00110101, $2, $4); canskip(); }
			| NEIW   wa ',' immb { op3(b01100101, $2, $4); canskip(); }
			| EQIW   wa ',' immb { op3(b01110101, $2, $4); canskip(); }
			| ONIW   wa ',' immb { op3(b01000101, $2, $4); canskip(); }
			| OFFIW  wa ',' immb { op3(b01010101, $2, $4); canskip(); }

			| EADD EA ',' r2    { op2(b01110000, b01000000 | $4); }
			| DADD EA ',' rp3   { op2(b01110100, b11000100 | $4); }
			| DADC EA ',' rp3   { op2(b01110100, b11010100 | $4); }
			| DADDNC EA ',' rp3 { op2(b01110100, b10100100 | $4); canskip(); }
			| ESUB EA ',' r2    { op2(b01110000, b01100000 | $4); }
			| DSUB EA ',' rp3   { op2(b01110100, b11100100 | $4); }
			| DSBB EA ',' rp3   { op2(b01110100, b11110100 | $4); }
			| DSUBNB EA ',' rp3 { op2(b01110100, b10110100 | $4); canskip(); }
			| DAN  EA ',' rp3   { op2(b01110100, b10001100 | $4); }
			| DOR  EA ',' rp3   { op2(b01110100, b10011100 | $4); }
			| DXR  EA ',' rp3   { op2(b01110100, b10010100 | $4); }
			| DGT  EA ',' rp3   { op2(b01110100, b10101100 | $4); canskip(); }
			| DLT  EA ',' rp3   { op2(b01110100, b10111100 | $4); canskip(); }
			| DNE  EA ',' rp3   { op2(b01110100, b11101100 | $4); canskip(); }
			| DEQ  EA ',' rp3   { op2(b01110100, b11111100 | $4); canskip(); }
			| DON  EA ',' rp3   { op2(b01110100, b11001100 | $4); canskip(); }
			| DOFF EA ',' rp3   { op2(b01110100, b11011100 | $4); canskip(); }

			| MUL r2 { op2(b01001000, b00101100 | $2); }
			| DIV r2 { op2(b01001000, b00111100 | $2); }

			| INR r2  { op1(b01000000 | $2); canskip(); }
			| INRW wa { op2(b00100000, $2); canskip(); }
			| INX rp  { op1(b00000010 | ($2 << 4)); }
			| INX EA  { op1(b10101000); }
			| DCR r2  { op1(b01010000 | $2); canskip(); }
			| DCRW wa { op2(b00110000, $2); canskip(); }
			| DCX rp  { op1(b00000011 | ($2 << 4)); }
			| DCX EA  { op1(b10101001); }

			| DAA  { op1(b01100001); }
			| STC  { op2(b01001000, b00101011); }
			| CLC  { op2(b01001000, b00101010); }
			| NEGA { op2(b01001000, b00111010); }

			| RLD     { op2(b01001000, b00111000); }
			| RRD     { op2(b01001000, b00111001); }
			| RLL  r2 { op2(b01001000, b00110100 | $2); }
			| RLR  r2 { op2(b01001000, b00110000 | $2); }
			| SLL  r2 { op2(b01001000, b00100100 | $2); }
			| SLR  r2 { op2(b01001000, b00100000 | $2); }
			| SLLC r2 { op2(b01001000, b00000100 | $2); canskip(); }
			| SLRC r2 { op2(b01001000, b00000000 | $2); canskip(); }

			| DRLL EA { op2(b01001000, b10110100); }
			| DRLR EA { op2(b01001000, b10110000); }
			| DSLL EA { op2(b01001000, b10100100); }
			| DSLR EA { op2(b01001000, b10100000); }

			| JMP  expr { op3(b01010100, $2, $2 >> 8); }
			| CALL expr { op3(b01000000, $2, $2 >> 8); }
			| JMP  BC   { op1(b00100001); }
			| CALL BC   { op2(b01001000, b00101001); }
			| JMP EA    { op2(b01001000, b00101000); }

			| JR expr  { int off = $2 - dot - 1;
			             if (pass && (off < -32 || off > 31))
							yyerror("Relative jump out of range");
						 op1(b11000000 | (off & 0x3F));
			}
			| JRE expr { int off = $2 - dot - 2;
			             if (pass && (off < -256 || off > 255))
							yyerror("Extended relative jump out of range");
						 op2(b01001110 | ((off & 0x1FF) >> 8), off);
			}
			| CALF expr { int fcn = $2;
						  if (pass && ((fcn & ~0x07FF) != 0x0800))
							yyerror("Fixed sub call out of range");
						  op2(b01111000 | ((fcn & 0x07FF) >> 8), fcn);
			}
			| CALT expr { int idx = $2;
						  if (pass && ((unsigned) idx >= 32))
							yyerror("Call table index out of range");
						  op1(b10000000 | idx);
			}
			| SOFTI { op1(b01110010); }
			| RET   { op1(b10111000); }
			| RETS  { op1(b10111001); /* doesskip(); */ }
			| RETI  { op1(b01100010); }

			| BIT bit ',' wa { op2(b01011000 | $2, $4); canskip(); }

			| SK    f     { op2(b01001000, b00001000 | $2); canskip(); }
			| SKN   f     { op2(b01001000, b00011000 | $2); canskip(); }
			| SKIT  irf   { op2(b01001000, b01000000 | $2); canskip(); }
			| SKNIT irf   { op2(b01001000, b01100000 | $2); canskip(); }

			| NOP  { op1(b00000000); }
			| EI   { op1(b10101010); }
			| DI   { op1(b10111010); }
			| HLT  { op2(b01001000, b00111011); }
			| STOP { op2(b01001000, b10111011); }

			;

%%

static char *vers2 = "@(#)uPD7810.y";

char *cpu_family = "uPD7810";
int MSB_first = 1; /* XXX fix reversal bug in emitw, not here */



