
!----------------------------------------------------------------------
!
!  NOKIA MOBIRA HSN-2 handset (at least -2D and -2F)
!
!  Fri Mar 10 20:21:03 EET 2000
!  Juha Nurmela
!
!  uPD7810 CPU
!  uPD7228 LCD
!  PCD3312 DTMF encoder
!  and other misc fluff
!
#define VERSION "HSN2 1.2  10.3.000"
!                12345678--12345678
!
!----------------------------------------------------------------------
!
#define MBUSID "HSN2"

!----------------------------------------------------------------------
!
!  We SEND each keypress in separate packet.
!
!    0 "ANY1" "HSN2" "M" "." \4         e.g. make of '1'
!    0 "ANY1" "HSN2" "R" "." \4         e.g. repeat of '1'
!
!  We accept following packets.
!
!    0 "HSN2" "...." "d" "........" \4  e.g. display "hepokatt"
!    0 "HSN2" "...." "f" "........" \4  e.g. display "  433500"
!    0 "HSN2" "...." "s" ".." \4        e.g. display signal "99"
!    0 "HSN2" "...." "v" "." \4         e.g. volume setting 0..15
!    0 "HSN2" "...." "l" "s?t?" \4      e.g. lightshow SERV, ALPHA, BACKLIGHT
!    0 "HSN2" "...." "b" "#555*" \4     e.g. signal DTMF, R is special
!
!----------------------------------------------------------------------
!
!  portpins

! PA, all outputs (VIEWCONT initially 0=high angle, could be input too)


SCK      = 0x01
VIEWCONT = 0x02 ! 3-way, 0=high angle, input=low angle, 1=; for lcd contrast
AUDIO    = 0x04 ! master enable for both EAR and SPKR
VOLUME   = 0x38
EAR      = 0x40 ! earphone
SPKR     = 0x80 ! internal speaker (and outgoing external line)

! PB, POWERSW and BUSY inputs

POWERSW  = 0x01 ! input, "0" if "OFF" pressed
POWERAMP = 0x02 ! audio PA supply
PWRSAVE  = 0x04 ! general audio supply shutdown
MOD      = 0x08 ! enable modulation output
MODSEL   = 0x70 ! select DTMF, MIC, EMIC or HFMIC
 DTMF  = 0x00
 MIC   = 0x10
 EXTM  = 0x20 ! aka EMIC
 HFMIC = 0x50 ! medium gain for now (0x30...0x70)

BUSY     = 0x80 ! /BUSY from LCD

! PC, HOOKIND input (SCL/SDA toggle output=0/input states)

NMIMASK  = 0x04 ! hold /NMI "1" when set
HOOKIND  = 0x08 ! input
SI       = 0x10 ! to LCD
NETFREE  = 0x20 ! input (with hw mode)
SCL      = 0x40
SDA      = 0x80

! Analog inputs AN0...3 in select mode
! AN1 nominated as the /PTT

!----------------------------------------------------------------------
!
!  external output latches (wronly, overlapping ronly eprom)
!
!  encode the address offset shifted by 2, bitmask in 2 LSbits

KBD_R0    = ((0 << 2) | 1)
KBD_R1    = ((1 << 2) | 1)
KBD_R2    = ((2 << 2) | 1)
KBD_R3    = ((3 << 2) | 1)
KBD_R4    = ((4 << 2) | 1)
KBD_R5    = ((5 << 2) | 1)
KBD_R6    = ((6 << 2) | 1)
BACKLIGHT = ((7 << 2) | 1)
LCD_CS    = ((0 << 2) | 2)
LCD_CD    = ((1 << 2) | 2)
LED_ON    = ((2 << 2) | 2)
LED_SERV  = ((3 << 2) | 2)
LED_ROAM  = ((4 << 2) | 2)
LED_CALL  = ((5 << 2) | 2)
LED_HF    = ((6 << 2) | 2)
LED_MFT   = ((7 << 2) | 2)

!----------------------------------------------------------------------

DTMF_ADDR = 0x4A  ! PCF3312 i2c address strap is "1"

!----------------------------------------------------------------------
!
!  LCD symbol characters are
!  F C N   < - >  T   A L P H A   key
!  We will use characters in FCN and ALPHA groups as a single symbols.
!
ICON_FCN   = 0x0202 ! (column_offset << 8) | (strlen(fcn) - 1)
ICON_LEFT  = 0x0901
ICON_RIGHT = 0x0A01 ! <- and -> overlap each other
ICON_T     = 0x0F00
ICON_ALPHA = 0x1404
ICON_KEY   = 0x1E00

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  external RAM 8192 bytes
!  stack is located at the end of the ram.
!  Remember, workarea only 256 bytes (V.wa)
!  DO NOT CHANGE V from workarea !

	.data
	.org 0xE000
ram:
workarea: ! XXX cook up some macros to give "safety" with LO(var)

latches:     .rs 8   ; ASSERT(LO(latches) == 0)

lcd_cursor:  .rs 1

mute_state:  .rs 1
volume:      .rs 1

kbd_bounce:  .rs 1   ! ticks to 0 whenever nonzero
kbd_repeat:  .rs 1   ! ticks to 255 whenever nonzero
kbd_flags:   .rs 1
 SHIFT   = 1
 ALPHA   = 2
 KEYLOCK = 4

cur_key:     .rs 1
ptt_state:   .rs 1

middle_row_buffer: .rs 8
bottom_row_buffer: .rs 8

	ASSERT(. <= workarea + 256)

#include "mbus.asm"

	! 3 kB of stack left. after above include
	.org 0xFFFE
stack:

!----------------------------------------------------------------------
!
!
!
	.text
!--------------------
	.org 0x0000     ! /RESET
	di
	jmp _start

!--------------------
	.org 0x0004     ! /NMI - wakeup!
	reti

!
!  Flags are not tested when entrypoints
!  have only one unmasked source.
!  The flag will be automatically cleared (when vectoring ?).
!
!--------------------
	.org 0x0008     ! INTT1 - 112 Hz (INTT0 not used)
	jmp systick

!--------------------
	.org 0x0010     ! INT1 /INT2 (INT1 is grounded, unused)
	jmp gone_off_hook

!--------------------
	.org 0x0018     ! INTE0 - timer/event counter, NETFREE indication
	jmp netfree

	.org 0x0020     ! INTEIN INTAD - CI/TO edge, A/D complete (unused)
	ei
	reti

	ASSERT(. <= 0x0028)
!--------------------
	.org 0x0028     ! INTST INTSR - serial port (both interrupts unmasked)
	jmp serint

	ASSERT(. <= 0x0060) ! catch overrun
!--------------------
	.org 0x0060     ! SOFTI - software interrupt (might be used)
	reti

!----------------------------------------------------------------------
!
!  Call table
!
#define CALT_REC(x) tab_/**/x = (. - 0x0080) / 2; .word x;
	ASSERT(. <= 0x0080) ! catch overrun
!--------------------
	.org 0x0080
	CALT_REC(latch_0)
	CALT_REC(latch_1)

	ASSERT(. <= 0x00C0) ! End of call table

!----------------------------------------------------------------------
!
!  interrupt functions, no register saved yet
!
gone_off_hook:
	ei
	reti

systick:
	exa
	mov A, [kbd_bounce]
	nei A, 0
	?jr 1f
	sui A, 1
	mov [kbd_bounce], A
1:
	mov A, [kbd_repeat]
	nei A, 0
	?jr 1f
	nei A, 0xFF
	?jr 1f
	adi A, 1
	mov [kbd_repeat], A
1:
	exa
	ei
	reti

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  MAIN LOOP
!

mainloop:

	neiw LO(kbd_bounce), 0
	?call scan_keys

	neiw LO(kbd_repeat), 255
	?call repeat_key

	lhld [mbus_rxwp]
	dmov EA, HL
	lhld [mbus_rxrp]
	deq EA, HL
	?call mbus_process

	call ptt

	! hlt

	jmp mainloop

!----------------------------------------------------------------------
ptt:
	mov A, CR1
	ani A, 0x80
	neaw LO(ptt_state)
	?ret
	staw LO(ptt_state)
	sll A
	mvi A, LED_ROAM
	sk CY
	?jmp latch_1
	jmp latch_0

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  STARTUP
!

_start:

	mvi A, 0
	mov MM, A       ! will just use the external RAM
	lxi SP, stack
	mvi V, HI(workarea) ! DO NOT CHANGE V from workarea

	lxi HL, ram
	lxi EA, 0
	mvi A, 0
1:
	stax [HL]+
	deq EA, HL
	?jr 1b

	call cpu_init
	call led_init
	call lcd_init
	call kbd_init
	call vol_set
	call mbus_init

	call enable_ints
	ei

	mvi A, BACKLIGHT
	call latch_1

	call dtmf_off

	call lcd_middle_row

	lxi HL, version
	call lcd_string

	jmp mainloop

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  CPU configuration

cpu_init:

	mvi PA, SCK | AUDIO | EAR | SPKR ! default VIEWCONT=0, lowest volume
	mvi A, 0
	mov MA, A            ! inputs 1

	mvi PB, POWERAMP | MOD | MIC
	mvi A, BUSY | POWERSW
	mov MB, A            ! inputs 1

	mvi PC, SCL | SDA | NMIMASK  ! unmask /nmi later if using powersaves
	mvi A, 0x23                  ! CI,RXD,TXD special
	mov MCC, A
	mvi A, NETFREE | HOOKIND | 0x02 ! last for RXD
	mov MC, A            ! inputs 1

	mvi ANM, 0x00  ! AN4..7 are used only thru edge flags. 0..3 in scan mode.

	mvi TMM, 0x20  ! TMR0 run from f/12 into F/F, TMR1 run from f/384
	mvi A, 3
	mov TM0, A     ! 921.6 kHz, to generate serial 9600 bd for MBUS
	mvi A, 0xFF
	mov TM1, A     ! to generate 112 Hz systick (will use like 100 Hz)

	mvi A, 0x5E
	mov SML, A     ! 1stop, odd parity, 8bits, x16 clock
	mvi A, 0
	mov SMH, A     ! flush
	mvi A, 0x0C
	mov SMH, A     ! rx enable, tx enable, internal TO clock

	mvi C, 7
1:
	skit INTFST    ! flush tx
	?nop
	skit INTFSR    ! flush rx
	?nop
	skit ER        ! flush rx
	?nop
	mov A, RXB     ! flush rx
	dcr C
	?jr 1b

	mvi A, 0
	mov EOM, A     ! not interested at PWM capabilities

	mvi A, 0x08
	mov ETMM, A    ! clear ECNT at CI falling edge, internal f/12 clock
	lxi EA, 1843
	dmov ETM0, EA  ! 921.6 kHz / ETM0 = 500 Hz, about 20 mbus chars.

	ret

enable_ints:

	ani MKL, ~0x24 ! INTE0, INTT1
	ani MKH, ~0x06 ! INTST, INTSR

	skit INTFT1
	?nop
	skit INTFE0
	?nop
	skit INTFST
	?nop
	skit INTFSR
	?nop

	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  Audio stuff
!

toggle_mute:
	mov A, [mute_state]
	adi A, 1
	lti A, 3
	?mvi A, 0
	mov [mute_state], A
	jmp mute_set

mute_set:
	mov A, [mute_state]
	eqi A, 0
	?jr 1f
	ori PB, POWERAMP       ! mute state 0
	ori PA, AUDIO | EAR | SPKR
	ret
1:
	eqi A, 1
	?jr 1f
	ani PB, ~POWERAMP      ! mute state 1
	ani PA, ~SPKR
	ori PA, AUDIO | EAR
	ret
1:
	eqi A, 2
	?jr 1f
	ani PB, ~POWERAMP      ! mute state 2
	ani PA, ~(AUDIO | EAR | SPKR)
	ret
1:
	ret

vol_inc:
	mov A, [volume]
	adi A, 1
	lti A, 8
	?mvi A, 7
	jr 1f

vol_dec:
	mov A, [volume]
	sui A, 1
	lti A, 8
	?mvi A, 0
1:
	mov [volume], A   ! FALLTHRU

vol_set:
	mov A, [volume]
	ani PA, ~VOLUME
	offi A, 1
	?ori PA, 0x08
	offi A, 2
	?ori PA, 0x10
	offi A, 4
	?ori PA, 0x20
	ret

keyclick:
	push VA
	mvi A,      1 ! click_vol XXX
	eqi A, 0
	?call 1f
	pop VA
	ret
1:
	ori PA, AUDIO | EAR
	sui A, 1
	ani PA, ~VOLUME
	offi A, 4
	?ori PA, 0x20
	offi A, 2
	?ori PA, 0x10
	offi A, 1
	?ori PA, 0x08
	mvi A, 0x3F      ! tone
	call i2c_dtmf
	mvi A, 0x00      ! notone
	call i2c_dtmf
	call vol_set
	call mute_set
	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  OUTPUT LATCHES

latch_0:
	push HL
	push BC
	call latch_proloq
	xri B, 0xFF      ! invert bitmask
	ana A, B
	jmp latch_epiloq

latch_1:
	push HL
	push BC
	call latch_proloq
	ora A, B
	jmp latch_epiloq

latch_toggle:
	push HL
	push BC
	call latch_proloq
	xra A, B
	jmp latch_epiloq

latch_proloq:
	mov B, A
	ani B, 3        ! bitmask
	slr A
	slr A
	mov L, A        ! offset
	mvi H, HI(latches) ; ASSERT(LO(latches) == 0)
	ldax [HL]
	ret
latch_epiloq:
	stax [HL]
	mvi H, 0        ! real latches at 0
	stax [HL]
	pop BC
	pop HL
	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  LEDS

led_init:
	mvi A, LED_ON
	call latch_1

	mvi A, LED_SERV
	call latch_0
	mvi A, LED_ROAM
	call latch_0
	mvi A, LED_CALL
	call latch_1       ! CALL until first mbus message for us.
	mvi A, LED_HF
	call latch_0
	mvi A, LED_MFT
	call latch_0

	mvi A, BACKLIGHT
	call latch_1
	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  DTMF
!
!  PC pin SDA is kept output always, so we ack ourselves =:I

i2c_bit:
	sll A
	skn CY
	?ori PC, SDA
	nop
	ori PC, SCL
	nop
	ani PC, ~SCL
	ani PC, ~SDA
	ret

i2c_byte:
	call i2c_bit
	call i2c_bit
	call i2c_bit
	call i2c_bit

	call i2c_bit
	call i2c_bit
	call i2c_bit
	call i2c_bit

	mvi A, 0
	call i2c_bit ! yucky, yucky...
	ret

i2c_start:
	ani PC, ~SDA ! START
	nop
	ani PC, ~SCL
	ret

i2c_stop:
	ori PC, SCL
	nop
	ori PC, SDA  ! STOP
	ret

i2c_dtmf:
	push VA
	call i2c_start

	mvi A, DTMF_ADDR
	call i2c_byte

	pop VA
	call i2c_byte

	call i2c_stop
	ret

dtmf_off:
	mvi A, 0x00 ! notone
	call i2c_dtmf
	ret

! for our own keys

dtmf_kbd_table:
	.byte '0', 0x10, '1', 0x11, '2', 0x12, '3', 0x13, '4', 0x14
	.byte '5', 0x15, '6', 0x16, '7', 0x17, '8', 0x18, '9', 0x19
	.byte '*', 0x1E, '#', 0x1F
	.byte 'F', 0x1A ! DTMF A
	.byte 'C', 0x1B ! DTMF B
	.byte 'E', 0x1C ! DTMF C
	.byte 'H', 0x1D ! DTMF D
	.byte 'A', 0x3F ! 1760 Hz
	.byte 0, 0

! for common ascii codes

dtmf_chr_table:
	.byte '0', 0x10, '1', 0x11, '2', 0x12, '3', 0x13, '4', 0x14
	.byte '5', 0x15, '6', 0x16, '7', 0x17, '8', 0x18, '9', 0x19
	.byte '*', 0x1E, '#', 0x1F
	.byte 'A', 0x1A ! DTMF A
	.byte 'B', 0x1B ! DTMF B
	.byte 'C', 0x1C ! DTMF C
	.byte 'D', 0x1D ! DTMF D
	.byte 'R', 0x3F ! 1760 Hz
	.byte 0, 0

dtmf_chr:
	push HL
	push BC
	mov C, A
	lxi HL, dtmf_chr_table
	jr 1f

dtmf_kbd:
	push HL
	push BC
	mov C, A
	lxi HL, dtmf_kbd_table
1:
	ldax [HL]+
	nei A, 0
	?jr 1f
	nea A, C
	?jr 1f
	inx HL
	jr 1b
1:
	ldax [HL]
	pop BC
	pop HL
	call i2c_dtmf
	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  LCD
!
!  CA=3 generates the middle 8 character cells and the top 2
!  small character cells [0...9]
!  CA=0 generates the bottom 8 character cells [10...17]

blanks: .asciz "                    " ! 20. enough.

lcd_init_seq_0:
1:
	.byte 0x18 ! - set multiplex mode (SYNC direction ???)
	.byte 0x12 ! - set frame freq
	.byte 0x09 ! - display on
	.byte 0x65 ! - set WRITE mode
	.byte 0xA7 ! - datapointer
lcd_init_size_0 = . - 1b

lcd_init_seq_3:
1:
	.byte 0x1A ! - set multiplex mode XXX (SYNC direction ???)
	.byte 0x12 ! - set frame frequency
	.byte 0x09 ! - display on
	.byte 0x65 ! - set WRITE mode
	.byte 0xB1 ! - datapointer
lcd_init_size_3 = . - 1b

lcd_init:

	call lcd_command_select_3

	lxi HL, lcd_init_seq_3
	mvi C, lcd_init_size_3
	call lcd_block

	call lcd_data_select_3
	mvi C, 49
1:
	mvi A, 0
	call lcd_byte          ! Empty out totally
	dcr C
	?jr 1b

	call lcd_command_select_0

	lxi HL, lcd_init_seq_0
	mvi C, lcd_init_size_0
	call lcd_block

	call lcd_data_select_0
	mvi C, 49
1:
	mvi A, 0
	call lcd_byte
	dcr C
	?jr 1b

	jmp lcd_middle_row     ! Leave cursor at middle row

lcd_data_mode:
	mvi A, LCD_CD
	call latch_0       ! /D
	call lcd_delay
	ret
lcd_command_mode:
	mvi A, LCD_CD
	call latch_1       ! C
	call lcd_delay
	ret
lcd_select:
	mvi A, LCD_CS
	call latch_0       ! /CS active
	call lcd_delay
	ret
lcd_unselect:
	mvi A, LCD_CS
	call latch_1       ! /CS high
	call lcd_delay
	ori PA, SCK        ! clock sense idle "1" (just in case)
	ret

lcd_data_select_3:
	call lcd_unselect
	call lcd_data_mode
	jr 1f

lcd_command_select_3:
	call lcd_unselect
	call lcd_command_mode
1:
	call lcd_select
	mvi A, 3
	call lcd_byte      ! CA
	ret

lcd_data_select_0:
	call lcd_unselect
	call lcd_data_mode
	jr 1f

lcd_command_select_0:
	call lcd_unselect
	call lcd_command_mode
1:
	call lcd_select
	mvi A, 0
	call lcd_byte      ! CA
	ret

lcd_delay:
	nop
	nop
	ret

lcd_bit:
	ani PA, ~SCK
	sllc A
	?ani PC, ~SI
	skn CY
	?ori PC, SI
	call lcd_delay
	ori PA, SCK
	ret

lcd_byte:
1:
	oni PB, BUSY
	?jr 1b          ! busywait LCD /BUSY

	call lcd_bit
	call lcd_bit
	call lcd_bit
	call lcd_bit

	call lcd_bit
	call lcd_bit
	call lcd_bit
	call lcd_bit

	ret

lcd_string:
1:
	ldax [HL]+
	nei A, 0
	?ret
	call lcd_chr
	jr 1b

! lcd_block is entered in the midst, it also does NOT walk cursor
! (command use)

1:
	ldax [HL]+
	call lcd_byte
lcd_block:
	dcr C
	?jr 1b
	ret

lcd_chr:
	call lcd_byte
	inrw LO(lcd_cursor)
	?nop
	neiw LO(lcd_cursor), 10
	?call lcd_bottom_row
	neiw LO(lcd_cursor), 18
	?call lcd_middle_row
	ret

lcd_digit:
	gti A, 9
	?adi A, '0'      ! not greater 9, change into ascii '0'...'9'
	gti A, 16
	?adi A, 'A' - 10 ! not yet ascii, change into 'A'... 'F'
	call lcd_chr
	ret

lcd_hex:
	push VA
	slr A
	slr A
	slr A
	slr A
	call lcd_digit
	pop VA
	ani A, 0x0F
	call lcd_digit
	ret

! BC has column offset and count of columns

lcd_symbol_onoff:
	eqi A, 0
	?jmp lcd_symbol_on
	jmp lcd_symbol_off

lcd_symbol_on:
	call lcd_command_select_3
	mvi A, 0xB1
	sub A, B                ! datapointer
	call lcd_byte
	mvi A, 0x69             ! - set OR mode
	call lcd_byte
	call lcd_data_select_3
1:
	mvi A, 0x80
	call lcd_byte
	dcr C
	?jr 1b

	jmp lcd_middle_row

lcd_symbol_off:
	call lcd_command_select_3
	mvi A, 0xB1
	sub A, B                ! datapointer
	call lcd_byte
	mvi A, 0x6D             ! - set AND mode
	call lcd_byte
	call lcd_data_select_3
1:
	mvi A, 0x7F
	call lcd_byte
	dcr C
	?jr 1b

	jmp lcd_middle_row

! 2 small cells at the top-right corner

lcd_top_row:

	call lcd_command_select_3
	mvi A, 0x71             ! - character mode, left entry
	call lcd_byte
	mvi A, 0xB1 - 8 * 5     ! - datapointer. past the middle row 8 cells
	call lcd_byte
	call lcd_data_select_3

	mviw LO(lcd_cursor), 8  ! past the middle row 8 cells
	ret

! 8 large cells in the middle

lcd_middle_row:

	call lcd_command_select_3
	mvi A, 0x71             ! - character mode, left entry
	call lcd_byte
	mvi A, 0xB1             ! - datapointer
	call lcd_byte
	call lcd_data_select_3

	mviw LO(lcd_cursor), 0
	ret

lcd_bottom_row:

	call lcd_command_select_0
	mvi A, 0x71             ! - character mode, left entry
	call lcd_byte
	mvi A, 0xA7             ! - datapointer
	call lcd_byte
	call lcd_data_select_0

	mviw LO(lcd_cursor), 10
	ret

! A has 0b10 for left, 0b00 for both off, 0b01 for right

icon_left_right:
	push BC
	push VA
	lxi BC, ICON_LEFT
	ani A, 0b10
	call lcd_symbol_onoff
	pop VA
	lxi BC, ICON_RIGHT
	ani A, 0b01
	call lcd_symbol_onoff
	pop BC
	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  KBD
!
!  scancode 0 is nokey, 1...0x1C are valid, some holes
!  POWERSW@PB translated to 'Q'


scancodes:
	.byte 0
	.byte 'M' ! 01 MUTE
	.byte 'O' ! 02 OK (LOCK)
	.byte 'N' ! 03 NEXT (PREV)
	.byte 'P' ! 04 PTT
	.byte 0
	.byte 'A' ! 06 R (ALPHA)
	.byte '?' ! 07 MENU (MFT)
	.byte 'R' ! 08 RCL (STO)
	.byte '3' ! 09 3 (GHI)
	.byte '2' ! 0A 2 (DEF)
	.byte '1' ! 0B 1 (ABC)
	.byte 'F' ! 0C FCN
	.byte '6' ! 0D 6 (PQR)
	.byte '5' ! 0E 5 (MNO)
	.byte '4' ! 0F 4 (JKL)
	.byte 'C' ! 10 CLR (CLA)
	.byte '9' ! 11 9 (YZ-)
	.byte '8' ! 12 8 (VWX)
	.byte '7' ! 13 7 (STU)
	.byte 'E' ! 14 luuri     (Earphone)
	.byte '#' ! 15 #
	.byte '0' ! 16 0 (AoA:O:)
	.byte '*' ! 17 *
	.byte 'H' ! 18 HF
	.byte 0
	.byte 0
	.byte '-' ! 1B -
	.byte '+' ! 1C +
	.byte 0
	.byte 0
	.byte 0
	.byte 0

kbd_seq:
	.byte KBD_R0, KBD_R1, KBD_R2, KBD_R3, KBD_R4, KBD_R5, KBD_R6

kbd_init:
	lxi HL, kbd_seq
	mvi C, 6         ! 7 rows
1:
	ldax [HL]+
	call latch_0
	dcr C
	?jr 1b
	skit AN4
	?nop
	skit AN5
	?nop
	skit AN6
	?nop
	skit AN7
	?nop
	ret

scan_keys:
	mvi D, 0         ! Pick current character to here

	oni PB, POWERSW
	?mvi D, 'Q'

	mvi B, 0         ! walk up scancode
	mvi C, 6         ! 7 lines
	lxi HL, kbd_seq
1:
	ldax [HL]
	call latch_1
	ldax [HL]+
	call latch_0     ! trailing edge rises ANx if contact closed

	inr B            ! B will not overflow, scancodes 1...29
	?sknit AN4
	?call kbd_hit
	inr B
	?sknit AN5
	?call kbd_hit
	inr B
	?sknit AN6
	?call kbd_hit
	inr B
	?sknit AN7
	?call kbd_hit
	dcr C
	?jre 1b

	mviw LO(kbd_bounce), 10 ! next stop here

	mov A, D
	neaw LO(cur_key)
	?ret
	staw LO(cur_key)        ! state changed, remember
	eqi A, 0
	?jmp got_key            ! no key into some key
	jmp no_key              ! some key into none

kbd_hit:
	push HL
	lxi HL, scancodes
	ldax [HL+B]
	mov D, A
	pop HL
	ret

!----------------------------------------------------------------------
!
!  Release of a key (none currently pressed)
!
no_key:
	mviw LO(kbd_repeat), 0
	call dtmf_off
	ret

!----------------------------------------------------------------------
!
!  New keyboard press, character in A
!
got_key:
	mviw LO(kbd_repeat), 255 - 112 ! typematic after 1 sec

	call keyclick

	neiw LO(ptt_state), 0
	?call dtmf_kbd

	mov A, [cur_key]
	mvi B, 'M'
	call mbus_sendkey

	ret

!----------------------------------------------------------------------
!
!  Typematic keyboard action, character in cur_key
!
repeat_key:
	mviw LO(kbd_repeat), 255 - 112 / 3 ! typematic rate 3 Hz

	mov A, [cur_key]
	mvi B, 'R'
	call mbus_sendkey

	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  MBUS
!
#include "mbus.asm" /* again to get the text part */

key_proto: .ascii "ANY1", MBUSID, "M*\4"
!                  0123   4567     89 0

mbus_sendkey:
	lxi HL, key_proto
	lxi DE, mbus_txmsg
	mvi C, SIZE(key_proto) - 1
	block
	lxi HL, mbus_txmsg
	stax [HL + 9]
	mov A, B
	stax [HL + 8]

	jmp mbus_send

!----------------------------------------------------------------------
mbus_newmsg:

	ldax [HL]+
	eqi A, 'H'
	?ret
	ldax [HL]+
	eqi A, 'S'
	?ret
	ldax [HL]+
	eqi A, 'N'
	?ret
	ldax [HL]+
	eqi A, '2'
	?ret

	mvi A, LED_CALL          ! ours. dim out 'roaming'. bah.
	call latch_0

	ldax [HL]+               ! sender at [HL] now
	nei A, 0xFF
	?ret
	ldax [HL]+
	nei A, 0xFF
	?ret
	ldax [HL]+
	nei A, 0xFF
	?ret
	ldax [HL]+
	nei A, 0xFF
	?ret

	ldax [HL]+     ! now after addresses, length at C, \4 not included

	nei A, 'd'
	?jmp msg_display
	nei A, 'f'
	?jmp msg_freq
	nei A, 's'
	?jmp msg_signal_report
	nei A, 'v'
	?jmp msg_volume
	nei A, 'l'
	?jmp msg_leds
	nei A, 'b'
	?jmp msg_beeps

	ret ! hmph. dunno what it is.

!----------------------------------------------------------------------
! HL points to message data after command designator character.
! Length of data in C
! e.g. HL->"433500", C=6

!    0 "HSN2" "...." "d" "........" \4  e.g. display "hepokatt"

msg_display:
	call lcd_middle_row
	lti C, 8
	?mvi C, 8            ! truncate text to 8
	push BC
	call lcd_block       ! leftjust
	pop BC
	mvi A, 8
	sub A, C
	lxi HL, blanks
	mov C, A
	call lcd_block       ! pad right
	ret

!    0 "HSN2" "...." "f" "........" \4  e.g. display "  433500"

msg_freq:
	call lcd_bottom_row
	lti C, 8
	?mvi C, 8            ! truncate text to 8
	mvi A, 8
	sub A, C
	push BC
	push HL
	lxi HL, blanks       ! pad left
	mov C, A
	call lcd_block
	pop HL
	pop BC
	call lcd_block       ! rightjust
	ret

!    0 "HSN2" "...." "s" ".." \4  e.g. display "99"

msg_signal_report:
	call lcd_top_row
	lti C, 2
	?mvi C, 2            ! truncate text to 2
	mvi A, 2
	sub A, C
	push BC
	push HL
	lxi HL, blanks       ! pad left
	mov C, A
	call lcd_block
	pop HL
	pop BC
	call lcd_block       ! rightjust
	ret

!    0 "HSN2" "...." "v" "." \4         e.g. volume setting 0..9

msg_volume:
	ldax [HL]+
	sui A, '0'
	lti A, 7
	?mvi A, 7
	mov [volume], A
	jmp vol_set

!    0 "HSN2" "...." "b" ".*" \4        e.g. beeps

msg_beeps:
	ldax [HL]+
	push HL
	push BC
	call 1f
	pop BC
	pop HL
	dcr C
	?jmp msg_beeps
	call dtmf_off
	ret
1:
	call dtmf_chr
	call nap
	ret
nap:
	hlt; hlt; hlt
	hlt; hlt; hlt
	hlt; hlt; hlt
	hlt
	ret

!    0 "HSN2" "...." "l" "sT" \4        e.g. markers SERV=off, ROAM=on

msg_leds:
	ldax [HL]+
	push HL
	push BC
	call 1f
	pop BC
	pop HL
	dcr C
	?jmp msg_leds
	ret
1:
	mov B, A              ! save aBoriginal
	mov C, A
	ani C, ~0x20          ! Capitalize it for table
	lxi HL, led_chars - 1
1:
	inx HL
	ldax [HL]+
	nei A, 0      ! not last.
	?jr 1f
	eqa A, C      ! same.
	?jr 1b
	jmp 3f        ! hit. 
1:
	lxi HL, lcd_chars - 2
1:
	inx HL        ! words in table
	inx HL
	ldax [HL]+
	nei A, 0      ! not last.
	?jr 1f
	eqa A, C      ! same.
	?jr 1b
	jmp 4f        ! hit.
1:
	ret           ! unknown symbol. ignore.
3:
	ldax [HL]
	offi B, 0x20  ! uppercase lights the led.
	?jmp latch_0
	jmp latch_1
4:
	ldeax [HL]
	mov A, B
	ani A, 0x20   ! extract case-bit
	xri A, 0x20   ! flip, uppercase lights the icon
	dmov BC, EA
	jmp lcd_symbol_onoff

led_chars:
	.byte 'S', LED_SERV
	.byte 'T', LED_ROAM     ! T for TX
	.byte 'H', LED_HF
	.byte 'M', LED_MFT
	.byte 'C', LED_CALL
	.byte 'B', BACKLIGHT
	.byte 0
lcd_chars:
	.byte 'F'; .word ICON_FCN
	.byte 'A'; .word ICON_ALPHA
	.byte 'K'; .word ICON_KEY
	.byte 0

!----------------------------------------------------------------------

	.ascii "@(#)"
version:
	.asciz VERSION
TheEnd:

